--[[
This controls the configuration settings (and display of a sample set of
actions based on those settings) for each ActionSetLayout.  Each
BarActionSetLayout is associated with an BarActionSetLayoutConfig, and each
BarActionSetLayoutConfig refers to that same BarActionSetLayout instance.
--]]
uab.classes.BarLayoutConfig = uab.mixins.aceOO.Class(IActionSetLayoutConfig, uab.mixins.aceEvent)
uab.classes.BarLayoutConfig.count = 0

function uab.classes.BarLayoutConfig.PlacementDisplayValue(value)
    if value == uab.constants.placement.top then
        return "Top"
    elseif value == uab.constants.placement.bottom then
        return "Bottom"
    elseif value == uab.constants.placement.left then
        return "Left"
    else
        return "Right"
    end
end

function uab.classes.BarLayoutConfig.GrowDirectionDisplayValue(value)
    if value == uab.constants.grow.up then
        return "Up"
    elseif value == uab.constants.grow.down then
        return "Down"
    elseif value == uab.constants.grow.left then
        return "Left"
    else
        return "Right"
    end
end

function uab.classes.BarLayoutConfig.prototype:init(layout, settings)
    uab.classes.BarLayoutConfig.super.prototype.init(self)

	self.frames = {}

	self.settings = settings

	self:SettingsChanged()

	uab.classes.BarLayoutConfig.count = uab.classes.BarLayoutConfig.count + 1
	self.number = uab.classes.BarLayoutConfig.count
end

function uab.classes.BarLayoutConfig.prototype:GetFrame(parent)
	if not self.frame then
	    self:CreateFrame(parent)
	end
	
	return self.frame
end

function uab.classes.BarLayoutConfig.prototype:SettingsChanged(settings)
	if settings then
		self.settings = settings
	end

    self.settings.overflowClockwise = (self.settings.overflowClockwise or "true")
    self.settings.placement = (self.settings.placement or uab.constants.placement.right)
    self.settings.growDirection = (self.settings.growDirection or uab.constants.grow.right)
    self.settings.growLimit = (self.settings.growLimit or 4)

	if self.frame then
		--[[
		Update the controls in the frame to show the new settings
		--]]
    	
        self:SetOverflowClockwise(self.settings.overflowClockwise)
    	self:SetPlacement(self.settings.placement)
    	self:SetGrowDirection(self.settings.growDirection)
    	self:SetGrowLimit(self.settings.growLimit)
	end
end

function uab.classes.BarLayoutConfig.prototype:UseDefaults(bypassNotification)
	if not self.settings then
	    self.settings = {}
	end

    self.settings.overflowClockwise = "true"
    self.settings.placement = uab.constants.placement.right
    self.settings.growDirection = uab.constants.grow.right
    self.settings.growLimit = 4

	self:SettingsChanged()
	
    if not bypassNotification then
        uab.main:LayoutSettingsChanged()
    end
end

function uab.classes.BarLayoutConfig.prototype:MenuOptionArgs()
	--[[
	Build an AceOptions table of the options to be used to configure this
	config instance.  This table is inserted into a parent menu as a set of
	options.  If there are several options, just define them at the top level
	of the returned table.  Do not create a 'group' at the top level to enclose
	the options.
	--]]
	
	local options = {
        growlimit = {
            type = 'range',
            name = "Grow Limit",
            desc = "Maximum number of buttons to place in each row before expanding to the next row",
            min = 1,
            max = 24,
            step = 1,
            isPercent = false,
            get = function()
                return self.settings.growLimit
            end,
            set = function(limit)
                self:SetGrowLimit(limit)
            end,
        },
        clockwise = {
            type = 'toggle',
            name = "Overflow clockwise",
            desc = "When a bar grows past the grow limit, add another row relative to the last bar in a clockwise direction",
            get = function()
                return self.settings.overflowClockwise == "true"
            end,
            set = function(v)
                if v then
					self:SetOverflowClockwise("true")
				else
				    self:SetOverflowClockwise("false")
                end
            end,
        },
        placement = {
            type = 'text',
            name = "First button placement",
            desc = "Placement of the first button, relative to the cancel button",
            get = function()
                return uab.classes.BarLayoutConfig.PlacementDisplayValue(self.settings.placement)
            end,
            set = function(placement)
                if placement == uab.classes.BarLayoutConfig.PlacementDisplayValue(uab.constants.placement.top) then
                    self:SetPlacement(uab.constants.placement.top)
				elseif placement == uab.classes.BarLayoutConfig.PlacementDisplayValue(uab.constants.placement.bottom) then
                	self:SetPlacement(uab.constants.placement.bottom)
				elseif placement == uab.classes.BarLayoutConfig.PlacementDisplayValue(uab.constants.placement.left) then
                	self:SetPlacement(uab.constants.placement.left)
				elseif placement == uab.classes.BarLayoutConfig.PlacementDisplayValue(uab.constants.placement.right) then
                	self:SetPlacement(uab.constants.placement.right)
				end
            end,

            validate = {
				uab.classes.BarLayoutConfig.PlacementDisplayValue(uab.constants.placement.top),
				uab.classes.BarLayoutConfig.PlacementDisplayValue(uab.constants.placement.bottom),
				uab.classes.BarLayoutConfig.PlacementDisplayValue(uab.constants.placement.left),
				uab.classes.BarLayoutConfig.PlacementDisplayValue(uab.constants.placement.right),
			},
        },
        direction = {
            type = 'text',
            name = "Grow direction",
            desc = "The direction in which new buttons are added in each row",
            get = function()
                return uab.classes.BarLayoutConfig.GrowDirectionDisplayValue(self.settings.growDirection)
            end,
            set = function(direction)
                if direction == uab.classes.BarLayoutConfig.GrowDirectionDisplayValue(uab.constants.grow.up) then
	                self:SetGrowDirection(uab.constants.grow.up)
				elseif direction == uab.classes.BarLayoutConfig.GrowDirectionDisplayValue(uab.constants.grow.down) then
	                self:SetGrowDirection(uab.constants.grow.down)
				elseif direction == uab.classes.BarLayoutConfig.GrowDirectionDisplayValue(uab.constants.grow.left) then
	                self:SetGrowDirection(uab.constants.grow.left)
				elseif direction == uab.classes.BarLayoutConfig.GrowDirectionDisplayValue(uab.constants.grow.right) then
	                self:SetGrowDirection(uab.constants.grow.right)
				end
            end,

            validate = {
				uab.classes.BarLayoutConfig.GrowDirectionDisplayValue(uab.constants.grow.up),
				uab.classes.BarLayoutConfig.GrowDirectionDisplayValue(uab.constants.grow.down),
				uab.classes.BarLayoutConfig.GrowDirectionDisplayValue(uab.constants.grow.left),
				uab.classes.BarLayoutConfig.GrowDirectionDisplayValue(uab.constants.grow.right),
			},
        },
	}
	
	return options
end

--[[---------------------------------------------------------------------------
Helper methods, to be used only by the class itself
-----------------------------------------------------------------------------]]

function uab.classes.BarLayoutConfig.prototype:CreateFrame(parent)
	self.frame = CreateFrame("Frame", "uabBarOptions"..self.number, parent or UIParent, "UAB_BarLayoutOptionsTemplate")

	self:SetupFrame()
end

function uab.classes.BarLayoutConfig.prototype:SetupFrame()
	self.frames = {}
	
	self.frames.placement = getglobal(self.frame:GetName().."Placement")
	self.frames.placement.config = self

    uab.classes.BarLayoutConfig.InitDropdown = self

    UIDropDownMenu_Initialize(self.frames.placement,
        function()
            if uab.classes.BarLayoutConfig.InitDropdown then
                uab.classes.BarLayoutConfig.InitDropdown:PlacementListInitialize()
            else
                this:GetParent().config:PlacementListInitialize()
            end
        end)
	UIDropDownMenu_SetWidth(110, self.frames.placement)
	UIDropDownMenu_SetButtonWidth(24, self.frames.placement)
	UIDropDownMenu_JustifyText("LEFT", self.frames.placement)

	self.frames.growDirection = getglobal(self.frame:GetName().."GrowDirection")
	self.frames.growDirection.config = self

    UIDropDownMenu_Initialize(self.frames.growDirection,
        function()
            if uab.classes.BarLayoutConfig.InitDropdown then
                uab.classes.BarLayoutConfig.InitDropdown:GrowDirectionListInitialize()
            else
                this:GetParent().config:GrowDirectionListInitialize()
            end
        end)
	UIDropDownMenu_SetWidth(110, self.frames.growDirection)
	UIDropDownMenu_SetButtonWidth(24, self.frames.growDirection)
	UIDropDownMenu_JustifyText("LEFT", self.frames.growDirection)

    uab.classes.BarLayoutConfig.InitDropdown = nil

	self.frames.growLimit = getglobal(self.frame:GetName().."GrowLimit")
	self.frames.growLimit.config = self
    
    getglobal(self.frames.growLimit:GetName().."Text"):SetText("Grow Limit")
    self.frames.growLimit:SetScript("OnValueChanged", function(self, value) self.config:GrowLimitChanged() end)
    getglobal(self.frames.growLimit:GetName().."High"):SetText()
    getglobal(self.frames.growLimit:GetName().."Low"):SetText()
    self.frames.growLimit:SetMinMaxValues(1, 24)
    self.frames.growLimit:SetValueStep(1)

	self.frames.overflowDirection = getglobal(self.frame:GetName().."OverflowDirection")
    self.frames.overflowDirection.config = self

	getglobal(self.frames.overflowDirection:GetName().."Text"):SetText("Overflow buttons clockwise")
	self.frames.overflowDirection:SetScript("OnClick", function(self, button, down) self.config:OverflowDirectionButtonClicked(self) end)

	self:SetOverflowClockwise(self.settings.overflowClockwise)
	self:SetPlacement(self.settings.placement)
	self:SetGrowDirection(self.settings.growDirection)
	self:SetGrowLimit(self.settings.growLimit)
end

function uab.classes.BarLayoutConfig.prototype:PlacementListInitialize()
    local index = 1
    local selected = nil

    local options = {}

    table.insert(options, {	text = "Top",
                            func = uab.DropDownSelect,
                            arg1 = self,
                            arg2 = { func = "SelectPlacement", arg1 = uab.constants.placement.top }})
    table.insert(options, {	text = "Right",
                            func = uab.DropDownSelect,
                            arg1 = self,
                            arg2 = { func = "SelectPlacement", arg1 = uab.constants.placement.right }})
    table.insert(options, {	text = "Bottom",
                            func = uab.DropDownSelect,
                            arg1 = self,
                            arg2 = { func = "SelectPlacement", arg1 = uab.constants.placement.bottom }})
    table.insert(options, {	text = "Left",
                            func = uab.DropDownSelect,
                            arg1 = self,
                            arg2 = { func = "SelectPlacement", arg1 = uab.constants.placement.left }})

    for index, data in pairs(options) do
        UIDropDownMenu_AddButton(data)
        if self.settings.placement == data.arg2.arg1 then
            selected = index
        end
    end

    UIDropDownMenu_SetSelectedID(self.frames.placement, selected)
end

function uab.classes.BarLayoutConfig.prototype:SelectPlacement(selector, selection)
    UIDropDownMenu_SetSelectedID(self.frames.placement, selector:GetID())
    self:SetPlacement(selection)
end

function uab.classes.BarLayoutConfig.prototype:SetPlacement(value)
    local notify = (value ~= self.settings.placement)

    self.settings.placement = value

	if self.frames.placement then
    	UIDropDownMenu_SetText(uab.classes.BarLayoutConfig.PlacementDisplayValue(value), self.frames.placement)
	end
	
    self:SynchronizeGrowPattern()

    if notify then
        uab.main:LayoutSettingsChanged()
    end
end

function uab.classes.BarLayoutConfig.prototype:GrowDirectionListInitialize()
    local index = 1
    local selected = nil

    local options = {}

    table.insert(options, {	text = "Up",
                            func = uab.DropDownSelect,
                            arg1 =  self,
                            arg2 = { func = "SelectGrowDirection", arg1 = uab.constants.grow.up }})
    table.insert(options, {	text = "Right",
                            func = uab.DropDownSelect,
                            arg1 =  self,
                            arg2 = { func = "SelectGrowDirection", arg1 = uab.constants.grow.right }})
    table.insert(options, {	text = "Down",
                            func = uab.DropDownSelect,
                            arg1 =  self,
                            arg2 = { func = "SelectGrowDirection", arg1 = uab.constants.grow.down }})
    table.insert(options, {	text = "Left",
                            func = uab.DropDownSelect,
                            arg1 =  self,
                            arg2 = { func = "SelectGrowDirection", arg1 = uab.constants.grow.left }})

    for index, data in pairs(options) do
        UIDropDownMenu_AddButton(data)
        if self.settings.growDirection == data.arg2.arg1 then
            selected = index
        end
    end

    UIDropDownMenu_SetSelectedID(self.frames.growDirection, selected)
end

function uab.classes.BarLayoutConfig.prototype:SelectGrowDirection(selector, selection)
    UIDropDownMenu_SetSelectedID(self.frames.growDirection, selector:GetID())
    self:SetGrowDirection(selection)
end

function uab.classes.BarLayoutConfig.prototype:SetGrowDirection(value)
    local notify = (value ~= self.settings.growDirection)

    self.settings.growDirection = value

	if self.frames.growDirection then
		UIDropDownMenu_SetText(uab.classes.BarLayoutConfig.GrowDirectionDisplayValue(value), self.frames.growDirection)
	end

    self:SynchronizeGrowPattern()

    if notify then
        uab.main:LayoutSettingsChanged()
    end
end

function uab.classes.BarLayoutConfig.prototype:GrowLimitChanged()
    local limit = self.frames.growLimit:GetValue()

    self:SetGrowLimit(limit, true)
end

function uab.classes.BarLayoutConfig.prototype:SetGrowLimit(value, skipFrameSetValue)
    local notify = (value ~= self.settings.growLimit)

    self.settings.growLimit = value
    
    if self.frames.growLimit then
        getglobal(self.frames.growLimit:GetName().."Text"):SetText("Grow Limit - "..self.settings.growLimit)
        if not skipFrameSetValue then
            self.frames.growLimit:SetValue(self.settings.growLimit)
        end
	end

    if notify then
        uab.main:LayoutSettingsChanged()
    end
end

function uab.classes.BarLayoutConfig.prototype:OverflowDirectionButtonClicked(frame)
    self:SetOverflowClockwise(frame:GetChecked())
end

function uab.classes.BarLayoutConfig.prototype:SetOverflowClockwise(value)
    local newval

    if value and (tonumber(value) == 1 or value == "true") then
        newval = "true"
    else
        newval = "false"
    end

    local notify = (newval ~= self.settings.overflowClockwise)

    self.settings.overflowClockwise = newval
    if self.frames.overflowDirection then
    	self.frames.overflowDirection:SetChecked(self.settings.overflowClockwise == "true")
	end

    if notify then
        uab.main:LayoutSettingsChanged()
    end
end

function uab.classes.BarLayoutConfig.prototype:EnableOverflowDirection()
    if self.frames.overflowDirection then
		self.frames.overflowDirection:Enable()
	end
end

function uab.classes.BarLayoutConfig.prototype:DisableOverflowDirection()
end

function uab.classes.BarLayoutConfig.prototype:SynchronizeGrowPattern()
    self:EnableOverflowDirection()
    	
    if self.settings.placement and self.settings.growDirection then
        if self.settings.placement == uab.constants.placement.top then
            if self.settings.growDirection == uab.constants.grow.down then
                self:SetGrowDirection(uab.constants.grow.up)
            elseif self.settings.growDirection == uab.constants.grow.right then
                self:SetOverflowClockwise("false")
                self:DisableOverflowDirection()
            elseif self.settings.growDirection == uab.constants.grow.left then
                self:SetOverflowClockwise("true")
                self:DisableOverflowDirection()
            end
        elseif self.settings.placement == uab.constants.placement.bottom then
            if self.settings.growDirection == uab.constants.grow.up then
                self:SetGrowDirection(uab.constants.grow.down)
            elseif self.settings.growDirection == uab.constants.grow.right then
                self:SetOverflowClockwise("true")
                self:DisableOverflowDirection()
            elseif self.settings.growDirection == uab.constants.grow.left then
                self:SetOverflowClockwise("false")
                self:DisableOverflowDirection()
            end
        elseif self.settings.placement == uab.constants.placement.left then
            if self.settings.growDirection == uab.constants.grow.right then
                self:SetGrowDirection(uab.constants.grow.left)
            elseif self.settings.growDirection == uab.constants.grow.up then
                self:SetOverflowClockwise("false")
                self:DisableOverflowDirection()
            elseif self.settings.growDirection == uab.constants.grow.down then
                self:SetOverflowClockwise("true")
                self:DisableOverflowDirection()
            end
        elseif self.settings.placement == uab.constants.placement.right then
            if self.settings.growDirection == uab.constants.grow.left then
                self:SetGrowDirection(uab.constants.grow.right)
            elseif self.settings.growDirection == uab.constants.grow.up then
                self:SetOverflowClockwise("true")
                self:DisableOverflowDirection()
            elseif self.settings.growDirection == uab.constants.grow.down then
                self:SetOverflowClockwise("false")
                self:DisableOverflowDirection()
            end
        end
    end
end

